@@ -1067,7 +1067,7 @@ brasero_medium_init_caps (BraseroMedium *self,
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	/* These special media don't support/need burnfree and simulation */
-	if (priv->info & BRASERO_MEDIUM_PLUS)
+	if (priv->info & (BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_BD))
 		return;
 
 	if (priv->info & BRASERO_MEDIUM_CD) {
@@ -1676,6 +1676,59 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_medium_track_set_leadout_DVDR_blank (BraseroMedium *self,
+					     BraseroDeviceHandle *handle,
+					     BraseroMediumTrack *leadout,
+					     BraseroScsiErrCode *code)
+{
+	BraseroScsiFormatCapacitiesHdr *hdr = NULL;
+	BraseroScsiMaxCapacityDesc *current;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	BRASERO_MEDIA_LOG ("Using fallback method for blank CDR to retrieve NWA and leadout information");
+
+	/* NWA is easy for blank DVD-Rs, it's 0. So far, so good... */
+	priv->next_wr_add = 0;
+
+	result = brasero_mmc2_read_format_capacities (handle,
+						      &hdr,
+						      &size,
+						      code);
+	if (result != BRASERO_SCSI_OK) {
+		g_free (hdr);
+
+		BRASERO_MEDIA_LOG ("READ FORMAT CAPACITIES failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	/* See if the media is already formatted which means for -R media that 
+	 * they are blank. */
+	current = hdr->max_caps;
+	if (current->type & BRASERO_SCSI_DESC_FORMATTED) {
+		BRASERO_MEDIA_LOG ("Unformatted medium");
+		g_free (hdr);
+		return BRASERO_BURN_ERR;
+	}
+		
+	BRASERO_MEDIA_LOG ("Unformatted medium");
+
+	/* of course it starts at 0 since it's empty */
+	leadout->start = 0;
+	leadout->blocks_num = BRASERO_GET_32 (current->blocks_num);
+
+	BRASERO_MEDIA_LOG ("Leadout (through READ FORMAT CAPACITIES): start = %llu size = %llu",
+			  leadout->start,
+			  leadout->blocks_num);
+
+	g_free (hdr);
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_medium_track_set_leadout_CDR_blank (BraseroMedium *self,
 					    BraseroDeviceHandle *handle,
@@ -1717,29 +1770,21 @@ brasero_medium_track_set_leadout_CDR_blank (BraseroMedium *self,
 }
 
 static BraseroBurnResult
-brasero_medium_track_set_leadout (BraseroMedium *self,
-				  BraseroDeviceHandle *handle,
-				  BraseroMediumTrack *leadout,
-				  BraseroScsiErrCode *code)
+brasero_medium_set_write_mode_page (BraseroMedium *self,
+				    BraseroDeviceHandle *handle,
+				    BraseroScsiErrCode *code)
 {
 	BraseroScsiModeData *data = NULL;
-	BraseroScsiTrackInfo track_info;
 	BraseroScsiWritePage *wrt_page;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
-	gint track_num;
 	int size;
 
-	BRASERO_MEDIA_LOG ("Retrieving NWA and leadout information");
+	BRASERO_MEDIA_LOG ("Setting write mode page");
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
-	if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
-		BRASERO_MEDIA_LOG ("Overwritable medium  => skipping");
-		return BRASERO_BURN_OK;
-	}
-
-	/* NOTE this works for CDR, DVDR+-, BDR-SRM */
+	/* NOTE: this works for CDR, DVDR+-, BDR-SRM */
 	/* make sure the current write mode is TAO. Otherwise the drive will
 	 * return the first sector of the pregap instead of the first user
 	 * accessible sector. */
@@ -1784,32 +1829,56 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 		if (result != BRASERO_SCSI_OK) {
 			BRASERO_MEDIA_LOG ("MODE SELECT failed");
 
-			/* This only for CD-R */
-			if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
-				return brasero_medium_track_set_leadout_CDR_blank (self,
-										   handle,
-										   leadout,
-										   code);
-
 			/* This isn't necessarily a problem! we better try */
-			//	return BRASERO_BURN_ERR;
+			return BRASERO_BURN_ERR;
 		}
 	}
 	else {
 		BRASERO_MEDIA_LOG ("MODE SENSE failed");
+		/* This isn't necessarily a problem! we better try the rest */
+		return BRASERO_BURN_ERR;
+	}
 
-		/* This only for CD-R */
-		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
+	return BRASERO_BURN_OK;
+}
+
+static BraseroBurnResult
+brasero_medium_track_set_leadout (BraseroMedium *self,
+				  BraseroDeviceHandle *handle,
+				  BraseroMediumTrack *leadout,
+				  BraseroScsiErrCode *code)
+{
+	BraseroScsiTrackInfo track_info;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	gint track_num;
+	int size;
+
+	BRASERO_MEDIA_LOG ("Retrieving NWA and leadout information");
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	if (BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
+		BRASERO_MEDIA_LOG ("Overwritable medium  => skipping");
+		return BRASERO_BURN_OK;
+	}
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR)) {
+		/* This is necessary to make sure nwa won't be the start of the
+		 * pregap if the current write mode is SAO with blank CDR.
+		 * Carry on even if it fails.
+		 * This can work with CD-R/W and DVD-R/W. + media don't use the
+		 * write mode page anyway. */
+		result = brasero_medium_set_write_mode_page (self, handle, code);
+		if (result == BRASERO_BURN_ERR
+		&&  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
 			return brasero_medium_track_set_leadout_CDR_blank (self,
 									   handle,
 									   leadout,
 									   code);
-
-		/* This isn't necessarily a problem! we better try the rest */
-		//	return BRASERO_BURN_ERR;
 	}
 
-	/* at this point we know the type of the disc that's why we set the 
+	/* At this point we know the type of the disc that's why we set the 
 	 * size according to this type. That may help to avoid outrange address
 	 * errors. */
 	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DUAL_L|BRASERO_MEDIUM_WRITABLE))
@@ -1837,14 +1906,20 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 					       &size,
 					       code);
 	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("READ TRACK INFO failed");
+
 		/* This only for CD-R */
 		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
 			return brasero_medium_track_set_leadout_CDR_blank (self,
 									   handle,
 									   leadout,
 									   code);
-
-		BRASERO_MEDIA_LOG ("READ TRACK INFO failed");
+		else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BLANK))
+			return brasero_medium_track_set_leadout_DVDR_blank (self,
+									    handle,
+									    leadout,
+									    code);
+			 
 		return BRASERO_BURN_ERR;
 	}
 
@@ -1863,6 +1938,13 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 		BRASERO_MEDIA_LOG ("Using track size %d", leadout->blocks_num);
 	}
 
+	if (!leadout->blocks_num
+	&&   BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BLANK))
+		return brasero_medium_track_set_leadout_DVDR_blank (self,
+								    handle,
+								    leadout,
+								    code);
+
 	BRASERO_MEDIA_LOG ("Leadout: start = %llu size = %llu",
 			  leadout->start,
 			  leadout->blocks_num);
@@ -2144,6 +2226,7 @@ brasero_medium_get_contents (BraseroMedium *self,
 			     BraseroScsiErrCode *code)
 {
 	int size;
+	BraseroBurnResult res;
 	BraseroScsiResult result;
 	BraseroMediumPrivate *priv;
 	BraseroScsiDiscInfoStd *info = NULL;
@@ -2168,7 +2251,7 @@ brasero_medium_get_contents (BraseroMedium *self,
 		BRASERO_MEDIA_LOG ("Disc id %i", BRASERO_GET_32 (info->disc_id));
 		priv->id = g_strdup_printf ("%d", BRASERO_GET_32 (info->disc_id));
 	}
-	else if (!(priv->info & BRASERO_MEDIUM_CD))
+	else if (priv->info & BRASERO_MEDIUM_DVD)
 		brasero_medium_get_DVD_id (self, handle, code);
 
 	if (info->erasable)
@@ -2177,10 +2260,10 @@ brasero_medium_get_contents (BraseroMedium *self,
 	priv->first_open_track = -1;
 
 	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
-		brasero_medium_set_blank (self,
-					  handle,
-					  BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
-					  code);
+		res = brasero_medium_set_blank (self,
+						handle,
+						BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
+						code);
 	}
 	else if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		if (!BRASERO_MEDIUM_RANDOM_WRITABLE (priv->info)) {
@@ -2190,27 +2273,27 @@ brasero_medium_get_contents (BraseroMedium *self,
 			priv->first_open_track = BRASERO_FIRST_TRACK_IN_LAST_SESSION (info);
 			BRASERO_MEDIA_LOG ("First track in last open session %d", priv->first_open_track);
 
-			result = brasero_medium_get_sessions_info (self, handle, code);
+			res = brasero_medium_get_sessions_info (self, handle, code);
 		}
 		else {
 			/* if that type of media is in incomplete state that
 			 * means it has just been formatted. And therefore it's
 			 * blank. */
-			brasero_medium_set_blank (self,
-						  handle,
-						  BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
-						  code);
+			res = brasero_medium_set_blank (self,
+							handle,
+							BRASERO_FIRST_TRACK_IN_LAST_SESSION (info),
+							code);
 		}
 	}
 	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
 		priv->info |= BRASERO_MEDIUM_CLOSED;
 		BRASERO_MEDIA_LOG ("Closed media");
 
-		result = brasero_medium_get_sessions_info (self, handle, code);
+		res = brasero_medium_get_sessions_info (self, handle, code);
 	}
 
 	g_free (info);
-	return result;
+	return res;
 }
 
 /**
@@ -3158,6 +3241,16 @@ brasero_medium_can_be_written (BraseroMedium *medium)
 	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL))
 		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
 
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM))
+		return (caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
+
+	/* All types of BD-R */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_WRITABLE))
+		return (caps & BRASERO_DRIVE_CAPS_BDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
+		return (caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
+
 	return FALSE;
 }
 
@@ -3202,6 +3295,12 @@ brasero_medium_can_be_rewritten (BraseroMedium *medium)
 	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL))
 		return (caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
 
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_RAM))
+		return (caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_BDRE))
+		return (caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
+
 	return FALSE;
 }
 
