@@ -783,6 +783,7 @@ brasero_track_get_data_excluded_source (BraseroTrack *track,
 
 BraseroBurnResult
 brasero_track_get_data_paths (BraseroTrack *track,
+			      gboolean use_joliet,
 			      const gchar *grafts_path,
 			      const gchar *excluded_path,
 			      const gchar *emptydir,
@@ -798,6 +799,7 @@ brasero_track_get_data_paths (BraseroTrack *track,
 	data = (BraseroTrackData *) track;
 	result = brasero_mkisofs_base_write_to_files (data->grafts,
 						      data->excluded,
+						      use_joliet,
 						      emptydir,
 						      videodir,
 						      grafts_path,
@@ -923,10 +925,8 @@ brasero_track_set_checksum (BraseroTrack *track,
 	BraseroBurnResult result = BRASERO_BURN_OK;
 
 	if (type == track->checksum_type
-	&& (type == BRASERO_CHECKSUM_MD5
-	||  type == BRASERO_CHECKSUM_SHA1
-	||  type == BRASERO_CHECKSUM_SHA256)
-	&&  strcmp (checksum, track->checksum))
+	&& (type == BRASERO_CHECKSUM_MD5 || type == BRASERO_CHECKSUM_SHA1 || type == BRASERO_CHECKSUM_SHA256)
+	&&  checksum && strcmp (checksum, track->checksum))
 		result = BRASERO_BURN_ERR;
 
 	if (track->checksum)
@@ -1024,65 +1024,67 @@ brasero_track_get_image_size (BraseroTrack *track,
 
 	/* Convenience function */
 	if (track->type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN) {
-		gchar *path;
+		gchar *uri;
 
 		if (!image->image)
 			return BRASERO_BURN_ERR;
 
-		path = brasero_track_get_localpath (image->image);
-		if (!brasero_image_format_get_iso_size (path, blocks, size, error)) {
-			g_free (path);
+		uri = brasero_track_get_uri (image->image);
+
+		if (!brasero_image_format_get_iso_size (uri, blocks, size, error)) {
+			g_free (uri);
 			return BRASERO_BURN_ERR;
 		}
-		g_free (path);
+		g_free (uri);
 
 		if (block_size)
 			*block_size = 2048;
 	}
 	else if (track->type.subtype.img_format == BRASERO_IMAGE_FORMAT_CLONE) {
-		gchar *path;
+		gchar *uri;
 
-		path = brasero_track_get_image_source (track, FALSE);
-		if (!path)
+		uri = brasero_track_get_image_source (track, TRUE);
+
+		if (!uri)
 			return BRASERO_BURN_ERR;
 
-		if (!brasero_image_format_get_clone_size (path, blocks, size, error)) {
-			g_free (path);
+		if (!brasero_image_format_get_clone_size (uri, blocks, size, error)) {
+			g_free (uri);
 			return BRASERO_BURN_ERR;
 		}
-		g_free (path);
+		g_free (uri);
 
 		if (block_size)
 			*block_size = 2448;
 	}
 	else if (track->type.subtype.img_format == BRASERO_IMAGE_FORMAT_CDRDAO) {
-		gchar *path;
+		gchar *uri;
 
 		if (!image->toc)
 			return BRASERO_BURN_ERR;
 
-		path = brasero_track_get_localpath (image->toc);
-		if (!brasero_image_format_get_cdrdao_size (path, blocks, size, error)) {
-			g_free (path);
+		uri = brasero_track_get_uri (image->toc);
+		if (!brasero_image_format_get_cdrdao_size (uri, blocks, size, error)) {
+			g_free (uri);
 			return BRASERO_BURN_ERR;
 		}
-		g_free (path);
+		g_free (uri);
 
 		if (block_size)
 			*block_size = 2352;
 	}
 	else if (track->type.subtype.img_format == BRASERO_IMAGE_FORMAT_CUE) {
-		gchar *path;
+		gchar *uri;
 
 		if (!image->toc)
 			return BRASERO_BURN_ERR;
 
-		path = brasero_track_get_localpath (image->toc);
-		if (!brasero_image_format_get_cue_size (path, blocks, size, error)) {
-			g_free (path);
+		uri = brasero_track_get_uri (image->toc);
+		if (!brasero_image_format_get_cue_size (uri, blocks, size, error)) {
+			g_free (uri);
 			return BRASERO_BURN_ERR;
 		}
-		g_free (path);
+		g_free (uri);
 
 		if (block_size)
 			*block_size = 2352;
